
from trac.core import implements, Component
from trac.perm import IPermissionPolicy

class ReporterCloseTicketPermissionPolicy(Component):
    """Provides a the user who reported a ticket with the ability to close the
       ticket and update the description and propertoes even if normally the
       permissions wouldn't allow this.

    Don't forget to add `ReporterCloseTicketPermissionPolicy` to the permission_policies
    option in [trac].  You will also need to allow the 'resolve' action to take place for 'TICKET_CHGPROP'.
    
    The workflow
    [trac]
    permission_policies = DefaultPermissionPolicy,LegacyAttachmentPolicy,ReporterCloseTicketPermissionPolicy
    """

    implements(IPermissionPolicy)

    # IPermissionPolicy methods
    def check_permission(self, action, username, resource, perm):
        if username != 'anonymous' and action in ['TICKET_CHGPROP', 'TICKET_EDIT_DESCRIPTION'] \
                and resource.realm == 'ticket' and resource.id is not None:
            db = self.env.get_db_cnx()
            cursor = db.cursor()
            cursor.execute("SELECT reporter, status FROM ticket WHERE id=%s",
                           (resource.id,))
            for reporter, status in cursor:
                if status != 'closed' and reporter == username:
                    return True

        return None

